
 DM 'CLE LOG; CLE OUT';
 ************************************************************************
  Septmeber 16, 2013
  Version 1
  
  
  THIS IS AN EXAMPLE OF A SAS PROGRAM THAT LINKS NHIS DATA FOR 2004 FORWARD, DOWNLOADED
  FROM THE NCHS WEBSITE, AND IHIS DATA IN SAS FILE FORMAT.

  NOTES:
    CERTAIN VARIABLES IN FOR LINKING, INCLUDING SRVY_YR HHX NON_INTV,         
  	IN NHIS DATA ARE NEEDED AND NOT TO BE CHANGED BEFORE THE MERGE 

  	THIS SAS FILE WILL WORK FOR LINKING HOUSEHOLD LEVEL FILE  
  	
For this program to work, users must replace the contents and brackets <  > 
 in each of the lines for:
 
 	- 3 LIBNAME statements in section 1
 	- 4 macro variables in subsection 1.1 - 1.4
	- 1 list of variable names in section 2 	

  	
      STEP 1: CHANGE DATA PATH AND FILE NAMES IN SECTION 1 AS NEEDED.
               SECTION 1.1: SPECIFY THE NAME OF IHIS FORMAT LIBRARY
               SECTION 1.2: SPECIFY THE NAME OF THE NHIS DATA (Merging file)
               SECTION 1.3: SPECIFY THE NAME OF THE IHIS DATA (Master file)
               SECTION 1.4: SPECIFY NAME OF FINAL MERGED DATA
	
      STEP 2: SPECIFY THE VARIABLES OF THE NHIS DATA TO BE KEPT IN SECTION 2 
    
      STEP 3: SAVE CHANGES TO SAS PROGRAM AND RUN
 *************************************************************************;


/***********************************************************************/
/*** SECTION 1: DATA PATH AND FILE NAMES; CHANGE FILE NAMES AS NEEDED***/
/***********************************************************************/

LIBNAME NHIS '<insert path to folder containing NHIS raw data here>';
LIBNAME IHIS '<insert path to folder containing IHIS data & format library here>';


/* Create macro variable NHISYEAR and set to year currently being processed;
replace 'YEAR' with the year of data you are merging, e.g., 2004. */
%let nhisyear=YEAR;



/* SECTION 1.1: Create macro variable fmtlibh to identify format library. 
   Replace <insert IHIS format library name here> with the name of your IHIS format library. 
   Remove < and > and leave no spaces in your specification.  */

%let fmtlibh=ihis.<insert IHIS format library name here>;


/* SECTION 1.2: Create macro variable nhisdsnameh and to identify NHIS data set to be used for merge. 
   Replace <insert NHIS raw data file name here> with the name of your NHIS file. 
   Remove < and > and leave no spaces in your specification. */

%let nhisdsnameh=nhis.<insert NHIS raw data file name here>;


/* SECTION 1.3: Create macro variable ihisdsnameh and to identify IHIS data set being processed (MASTER FILE). 
   Replace <insert IHIS data file name here> with the name of your IHIS file. 
   Remove < and > and leave no spaces in your specification.*/

%let ihisdsnameh=ihis.<insert IHIS raw data file name here>;


/* SECTION 1.4: Create macro variable dsmergeh to identify final merged data. 
   Replace <insert merge file name here> with the name of your final merged IHIS data file. 
   Remove < and > and leave no spaces in your specification.  */

%let dsmergeh=ihis.<insert merge file name here>;

/***********************************************************************/
/*** SECTION 2: SPECIFY THE VARIABLES OF THE NHIS DATA TO BE KEPT IN****/
/***********************************************************************/

/* Replace <insert names of NHIS variables you would like to merge here> with the names  
   of the NHIS variables you want to merge to IHIS. 
   Separate the variables in your list with a space. Remove the < and >  */

%let indvarsh = <insert names of NHIS variables you would like to merge here>;






/***********************************************************************/
/***********************************************************************/
/***********************************************************************/



/*************************************************************/
/*** SECTION 3: PREPARING NHIS DATA TO MERGE WITH IHIS DATA***/
/*************************************************************/

/* SECTION 3.1: CHECK DUPLICATES FOR UNIQUE IDENTIFIER IN NHIS DATA */

PROC SORT DATA=&nhisdsnameh (keep=hhx)  OUT=A1 NODUPKEY;
BY hhx;
RUN;


/* SECTION 3.2: GENERATE LINKING KEY */

OPTIONS NOFMTERR;
DATA household&nhisyear;
  SET &nhisdsnameh;
	ATTRIB nhishid length=$14 LABEL='NHIS Household Unique Indentifier Key'
	       year length=4 LABEL='NHIS Year';
    year=&nhisyear;

nhishid=compress("0000"||put(srvy_yr,4.)||hhx);
RUN;

PROC SORT DATA=household&nhisyear;
BY year nhishid;
RUN;


/* SECTION 3.3: CHECK DUPLICATES LINKING IDENTIFIER IN NHIS DATA */

OPTIONS FULLSTIMER;
PROC SQL;
 TITLE 'CHECK DUPLICATES LINKING IDENTIFIER IN NHIS DATA' ;
 SELECT count(*) 'Total Records' AS total,
         count(DISTINCT cats(year,nhishid)) 'Unique Records' AS unique,
         calculated total- calculated unique 'Duplicates'
         INTO :totalrecs, :uniquerecs, :duprecs
     FROM household&nhisyear;
QUIT;

%PUT ****************************************************;
%PUT * NHIS duplicates check results:;
%PUT *   Total number of records in NHIS data set: &totalrecs;
%PUT *   Unique keys in NHIS data set: &uniquerecs;
%PUT *   Records that have a duplicate key: &duprecs;
%PUT ****************************************************;






/* SECTION 3.4: KEEP VARIABLES IN NHIS DATA */

OPTIONS NOFMTERR;
DATA household1&nhisyear;
  SET household&nhisyear;
  if non_intv=. then completed=1;
  else completed=2;
  KEEP year nhishid completed &indvarsh;
RUN;


/*************************************************/
/*** SECTION 4: MERGE NHIS DATA WITH IHIS DATA ***/
/*************************************************/

/* SECTION 4.1: CHECK DUPLICATES OF LINKING KEY IN IHIS DATA */

PROC FORMAT cntlin=&fmtlibh; 
RUN;
DATA ihis&nhisyear;
  SET &ihisdsnameh;
RUN; 

PROC SORT DATA=ihis&nhisyear;
  BY year nhishid;
RUN;

PROC SORT DATA=ihis&nhisyear(KEEP=year nhishid) OUT=A2 NODUPKEY;
  BY year nhishid;
RUN;


/* SECTION 4.2: MERGE NHIS DATA WITH IHIS DATA */

DATA &dsmergeh ;
   MERGE ihis&nhisyear(in=in1)  household1&nhisyear(in=in2);
   BY year nhishid;
   
ATTRIB _merge LENGTH=3 LABEL='DATA SET SOURCE FOR OBS';
   IF in1 and in2 then _merge=3;
   ELSE IF in1 and not in2 THEN _merge=1;
   ELSE IF not in1 and in2 THEN _merge=2;
RUN;    


/* SECTION 4.3: CHECK DUPLICATES OF LINKING KEY IN MERGE DATA  */

OPTIONS FULLSTIMER;
PROC SQL;
 TITLE 'DUPLICATES CHECK OF MERGED DATA SET';
 SELECT count(*) 'Total Records' AS total,
         count(DISTINCT cats(year,nhishid)) 'Unique Records' AS unique,
         calculated total- calculated unique 'Duplicates'
         INTO :totalrecs, :uniquerecs, :duprecs
     FROM &dsmergeh;
QUIT;

%PUT ****************************************************;
%PUT * MERGE DATA duplicates check results:;
%PUT *   Total number of records in merged data set: &totalrecs;
%PUT *   Unique keys in merged data set: &uniquerecs;
%PUT *   Records that have a duplicate key: &duprecs;
%PUT ****************************************************;


/* SECTION 4.4: CHECK THE RESULTS OF THE MERGE IN THE SAS LOG  */

TITLE 'CHECK THE RESULTS OF THE MERGE DATA';

PROC FREQ DATA=&dsmergeh;
  TABLES year*_merge*completed / NOROW NOCOL NOPERCENT OUT=__freqs;
RUN;
PROC TRANSPOSE DATA=__freqs
               OUT=__freqs2(drop=_name_ _label_)
               PREFIX=completed;
  BY year _merge;
  ID completed;
  VAR count;
RUN;
DATA _NULL_;
  SET __freqs2;
  BY year;

  FILE log;
  RETAIN yearcum1 yearcum2 0;

  IF first.year THEN DO;
    yearcum1=0;
    yearcum2=0;

    PUT // 'CHECK THE RESULTS OF THE MERGE DATA'/
           '                                   '/
		   'year=' year /
           '           |       completed '/
           '    _merge |       1         2 |    Total'/
           '-----------------------------------------';
  END;

  IF completed1=. THEN completed1=0;
  IF completed2=. THEN completed2=0;

  yearcum1+completed1;
  yearcum2+completed2;

  _mergesum=sum(completed1,completed2);

  PUT @10 _merge '| '
      @13 completed1 8. '  '
      @23 completed2 8. ' |'
      @33 _mergesum  8.;

  IF last.year THEN DO;
    total=sum(yearcum1,yearcum2);
    PUT '-----------------------------------------' /
        @6 'Total | ' @13 yearcum1 8. '  ' @23 yearcum2 8. ' |' total 8.;
  END;
RUN;
